@@ -1,6 +1,6 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
-// Copyright Bob Friesenhahn, 1999, 2000, 2001
+// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002, 2003
 //
 // Implementation of Options
 //
@@ -9,22 +9,20 @@
 
 #define MAGICK_IMPLEMENTATION
 
+#include "Magick++/Include.h"
 #include <string>
+#include <string.h>
 #include <stdlib.h>
-#include <iostream>
 #include <math.h>
 #include "Magick++/Options.h"
 #include "Magick++/Functions.h"
 #include "Magick++/Exception.h"
-#include "Magick++/Include.h"
-
-#define DegreesToRadians(x) ((x)*3.14159265358979323846/180.0)
 
 // Constructor
 Magick::Options::Options( void )
-  : _imageInfo(static_cast<ImageInfo*>(AcquireMemory(sizeof(ImageInfo)))),
-    _quantizeInfo(static_cast<QuantizeInfo*>(AcquireMemory(sizeof(QuantizeInfo)))),
-    _drawInfo(static_cast<DrawInfo*>(AcquireMemory( sizeof(DrawInfo))))
+  : _imageInfo(static_cast<ImageInfo*>(AcquireMagickMemory(sizeof(ImageInfo)))),
+    _quantizeInfo(static_cast<QuantizeInfo*>(AcquireMagickMemory(sizeof(QuantizeInfo)))),
+    _drawInfo(static_cast<DrawInfo*>(AcquireMagickMemory( sizeof(DrawInfo))))
 {
   // Initialize image info with defaults
   GetImageInfo( _imageInfo );
@@ -62,12 +60,15 @@ Magick::Options::~Options()
 {
   // Destroy image info
   DestroyImageInfo( _imageInfo );
-  
+  _imageInfo=0;
+
   // Destroy quantization info
   DestroyQuantizeInfo( _quantizeInfo );
+  _quantizeInfo=0;
 
   // Destroy drawing info
   DestroyDrawInfo( _drawInfo );
+  _drawInfo=0;
 }
 
 /*
@@ -75,6 +76,27 @@ Magick::Options::~Options()
  *
  */
 
+// Anti-alias Postscript and TrueType fonts (default true)
+void Magick::Options::antiAlias( bool flag_ )
+{
+  _drawInfo->text_antialias = static_cast<MagickBooleanType>
+    (flag_ ? MagickTrue : MagickFalse);
+}
+bool Magick::Options::antiAlias( void ) const
+{
+  return static_cast<bool>(_drawInfo->text_antialias);
+}
+
+void Magick::Options::adjoin ( bool flag_ )
+{
+  _imageInfo->adjoin = static_cast<MagickBooleanType>
+    (flag_ ? MagickTrue : MagickFalse);
+}
+bool Magick::Options::adjoin ( void ) const
+{
+  return static_cast<bool>(_imageInfo->adjoin);
+}
+
 void Magick::Options::backgroundColor ( const Magick::Color &color_ )
 {
   _imageInfo->background_color = color_;
@@ -87,7 +109,7 @@ Magick::Color Magick::Options::backgroundColor ( void ) const
 void Magick::Options::backgroundTexture ( const std::string &backgroundTexture_ )
 {
   if ( backgroundTexture_.length() == 0 )
-    LiberateMemory( reinterpret_cast<void**>(&_imageInfo->texture) );
+    _imageInfo->texture=(char *) RelinquishMagickMemory(_imageInfo->texture);
   else
     Magick::CloneString( &_imageInfo->texture, backgroundTexture_ );
 }
@@ -112,40 +134,56 @@ Magick::Color Magick::Options::borderColor ( void ) const
 // Text bounding-box base color
 void Magick::Options::boxColor ( const Magick::Color &boxColor_ )
 {
-  _drawInfo->box = boxColor_;
+  _drawInfo->undercolor = boxColor_;
 }
 Magick::Color Magick::Options::boxColor ( void ) const
 {
-  return Magick::Color( _drawInfo->box );
+  return Magick::Color( _drawInfo->undercolor );
 }
 
-void Magick::Options::strokeDashArray ( const double* strokeDashArray_ )
+void Magick::Options::compressType ( CompressionType compressType_ )
 {
-  LiberateMemory(reinterpret_cast<void**>(&_drawInfo->dash_pattern));
-	_drawInfo->dash_pattern = NULL;
+  _imageInfo->compression = compressType_;
+}
+Magick::CompressionType Magick::Options::compressType ( void ) const
+{
+  return static_cast<Magick::CompressionType>(_imageInfo->compression);
+}
 
-  if(strokeDashArray_)
+void Magick::Options::colorFuzz ( double fuzz_ )
+{
+  _imageInfo->fuzz = fuzz_;
+}
+double Magick::Options::colorFuzz ( void ) const
+{
+  return _imageInfo->fuzz;
+}
+
+// Enable printing of debug messages from ImageMagick
+void Magick::Options::debug ( bool flag_ )
+{
+  if(flag_)
     {
-      // Count elements in dash array
-      int x;
-      for (x=0; strokeDashArray_[x]; x++);
-      // Allocate elements
-      _drawInfo->dash_pattern =
-        static_cast<double*>(AcquireMemory((x+1)*sizeof(double)));
-      // Copy elements
-      memcpy(_drawInfo->dash_pattern,strokeDashArray_,
-             (x+1)*sizeof(double));
+      SetLogEventMask("All");
+    }
+  else
+    {
+      SetLogEventMask("None");
     }
 }
-const double* Magick::Options::strokeDashArray ( void ) const
+bool Magick::Options::debug ( void ) const
 {
-  return _drawInfo->dash_pattern;
+  if( IsEventLogging() )
+    {
+      return true;
+    }
+  return false;
 }
 
 void Magick::Options::density ( const Magick::Geometry &density_ )
 {
   if ( !density_.isValid() )
-    LiberateMemory( reinterpret_cast<void**>(&_imageInfo->density) );
+    _imageInfo->density=(char *) RelinquishMagickMemory(_imageInfo->density);
   else
     Magick::CloneString( &_imageInfo->density, density_ );
 }
@@ -157,6 +195,15 @@ Magick::Geometry Magick::Options::density ( void ) const
   return Geometry();
 }
 
+void Magick::Options::depth ( unsigned int depth_ )
+{
+  _imageInfo->depth = depth_;
+}
+unsigned int Magick::Options::depth ( void ) const
+{
+  return _imageInfo->depth;
+}
+
 // Endianness (little like Intel or big like SPARC) for image
 // formats which support endian-specific options.
 void Magick::Options::endian ( Magick::EndianType endian_ )
@@ -205,7 +252,7 @@ void Magick::Options::fillPattern ( const MagickLib::Image *fillPattern_ )
 	CloneImage( const_cast<MagickLib::Image*>(fillPattern_),
 		    0,
 		    0,
-		    static_cast<int>(true),
+		    static_cast<MagickBooleanType>(MagickTrue),
 		    &exceptionInfo );
       throwException( exceptionInfo );
     }
@@ -229,8 +276,8 @@ void Magick::Options::font ( const std::string &font_ )
 {
   if ( font_.length() == 0 )
     {
-      LiberateMemory( reinterpret_cast<void**>(&_imageInfo->font) );
-      LiberateMemory( reinterpret_cast<void**>(&_drawInfo->font) );
+      _imageInfo->font=(char *) RelinquishMagickMemory(_imageInfo->font);
+      _drawInfo->font=(char *) RelinquishMagickMemory(_drawInfo->font);
     }
   else
     {
@@ -246,13 +293,23 @@ std::string Magick::Options::font ( void ) const
   return std::string();
 }
 
+void Magick::Options::fontPointsize ( double pointSize_ )
+{
+  _imageInfo->pointsize = pointSize_;
+  _drawInfo->pointsize = pointSize_;
+}
+double Magick::Options::fontPointsize ( void ) const
+{
+  return _imageInfo->pointsize;
+}
+
 std::string Magick::Options::format ( void ) const
 {
   ExceptionInfo exception;
 
   const MagickInfo * magick_info = 0;
   GetExceptionInfo(&exception);
-  if ( _imageInfo->magick && ( *_imageInfo->magick != '\0' ))
+  if ( *_imageInfo->magick != '\0' )
     magick_info = GetMagickInfo( _imageInfo->magick , &exception);
   
   if (( magick_info != 0 ) && 
@@ -262,14 +319,23 @@ std::string Magick::Options::format ( void ) const
   return std::string();
 }
 
+void Magick::Options::interlaceType ( Magick::InterlaceType interlace_ )
+{
+  _imageInfo->interlace = interlace_;
+}
+Magick::InterlaceType Magick::Options::interlaceType ( void ) const
+{
+  return static_cast<Magick::InterlaceType>(_imageInfo->interlace);
+}
+
 void Magick::Options::magick ( const std::string &magick_ )
 {
   ExceptionInfo exception;
 
   FormatString( _imageInfo->filename, "%.1024s:", magick_.c_str() );
   GetExceptionInfo(&exception);
-  SetImageInfo( _imageInfo, 1, &exception);
-  if ( _imageInfo->magick == '\0' )
+  SetImageInfo( _imageInfo, MagickTrue, &exception);
+  if ( *_imageInfo->magick == '\0' )
     throwExceptionExplicit( OptionWarning,
 			    "Unrecognized image format",
 			    magick_.c_str() );
@@ -291,10 +357,19 @@ Magick::Color Magick::Options::matteColor ( void ) const
   return Magick::Color( _imageInfo->matte_color );
 }
 
+void Magick::Options::monochrome ( bool monochromeFlag_ )
+{
+  _imageInfo->monochrome = (MagickBooleanType) monochromeFlag_;
+}
+bool Magick::Options::monochrome ( void ) const
+{
+  return static_cast<bool>(_imageInfo->monochrome);
+}
+
 void Magick::Options::page ( const Magick::Geometry &pageSize_ )
 {
   if ( !pageSize_.isValid() )
-    LiberateMemory( reinterpret_cast<void**>(&_imageInfo->page) );
+    _imageInfo->page=(char *) RelinquishMagickMemory(_imageInfo->page);
   else
     Magick::CloneString( &_imageInfo->page, pageSize_ );
 }
@@ -306,6 +381,52 @@ Magick::Geometry Magick::Options::page ( void ) const
     return Geometry();
 }
 
+void Magick::Options::quality ( unsigned int quality_ )
+{
+  _imageInfo->quality = quality_;
+}
+unsigned int Magick::Options::quality ( void ) const
+{
+  return _imageInfo->quality;
+}
+
+void Magick::Options::quantizeColors ( unsigned int colors_ )
+{
+  _quantizeInfo->number_colors = colors_;
+}
+unsigned int Magick::Options::quantizeColors ( void ) const
+{
+  return _quantizeInfo->number_colors;
+}
+
+void Magick::Options::quantizeColorSpace ( Magick::ColorspaceType colorSpace_ )
+{
+  _quantizeInfo->colorspace = colorSpace_;
+}
+Magick::ColorspaceType Magick::Options::quantizeColorSpace ( void ) const
+{
+  return static_cast<Magick::ColorspaceType>(_quantizeInfo->colorspace);
+}
+
+void Magick::Options::quantizeDither ( bool ditherFlag_ )
+{
+  _imageInfo->dither = (MagickBooleanType) ditherFlag_;
+  _quantizeInfo->dither = (MagickBooleanType) ditherFlag_;
+}
+bool Magick::Options::quantizeDither ( void ) const
+{
+  return static_cast<bool>(_imageInfo->dither);
+}
+
+void Magick::Options::quantizeTreeDepth ( unsigned int treeDepth_ )
+{
+  _quantizeInfo->tree_depth = treeDepth_;
+}
+unsigned int Magick::Options::quantizeTreeDepth ( void ) const
+{
+  return _quantizeInfo->tree_depth;
+}
+
 void Magick::Options::resolutionUnits ( Magick::ResolutionType resolutionUnits_ )
 {
   _imageInfo->units = resolutionUnits_;
@@ -317,7 +438,7 @@ Magick::ResolutionType Magick::Options::resolutionUnits ( void ) const
 
 void Magick::Options::size ( const Geometry &geometry_ )
 {
-  LiberateMemory( reinterpret_cast<void**>(&_imageInfo->size) );
+  _imageInfo->size=(char *) RelinquishMagickMemory(_imageInfo->size);
 
   if ( geometry_.isValid() )
     Magick::CloneString( &_imageInfo->size, geometry_ );
@@ -330,6 +451,15 @@ Magick::Geometry Magick::Options::size ( void ) const
   return Geometry();
 }
 
+void Magick::Options::strokeAntiAlias( bool flag_ )
+{
+  flag_ ? _drawInfo->stroke_antialias=MagickTrue : _drawInfo->stroke_antialias=MagickFalse;
+}
+bool Magick::Options::strokeAntiAlias( void ) const
+{
+  return (_drawInfo->stroke_antialias != 0 ? true : false);
+}
+
 // Color to use when drawing object outlines
 void Magick::Options::strokeColor ( const Magick::Color &strokeColor_ )
 {
@@ -340,6 +470,71 @@ Magick::Color Magick::Options::strokeColor ( void ) const
   return _drawInfo->stroke;
 }
 
+void Magick::Options::strokeDashArray ( const double* strokeDashArray_ )
+{
+  _drawInfo->dash_pattern=(double *)
+    RelinquishMagickMemory(_drawInfo->dash_pattern);
+
+  if(strokeDashArray_)
+    {
+      // Count elements in dash array
+      unsigned int x;
+      for (x=0; strokeDashArray_[x]; x++);
+      // Allocate elements
+      _drawInfo->dash_pattern =
+        static_cast<double*>(AcquireMagickMemory((x+1)*sizeof(double)));
+      // Copy elements
+      memcpy(_drawInfo->dash_pattern,strokeDashArray_,
+             (x+1)*sizeof(double));
+    }
+}
+const double* Magick::Options::strokeDashArray ( void ) const
+{
+  return _drawInfo->dash_pattern;
+}
+
+void Magick::Options::strokeDashOffset ( double strokeDashOffset_ )
+{
+  _drawInfo->dash_offset = strokeDashOffset_;
+}
+double Magick::Options::strokeDashOffset ( void ) const
+{
+  return _drawInfo->dash_offset;
+}
+
+// Specify the shape to be used at the end of open subpaths when they
+// are stroked. Values of LineCap are ButtCap, RoundCap, and
+// SquareCap.
+void Magick::Options::strokeLineCap ( Magick::LineCap lineCap_ )
+{
+  _drawInfo->linecap = lineCap_;
+}
+Magick::LineCap Magick::Options::strokeLineCap ( void ) const
+{
+  return _drawInfo->linecap;
+}
+
+// Specify the shape to be used at the corners of paths (or other
+// vector shapes) when they are stroked.
+void Magick::Options::strokeLineJoin ( Magick::LineJoin lineJoin_ )
+{
+  _drawInfo->linejoin = lineJoin_;
+}
+Magick::LineJoin Magick::Options::strokeLineJoin ( void ) const
+{
+  return _drawInfo->linejoin;
+}
+
+// miterLimit for drawing lines, circles, ellipses, etc.
+void Magick::Options::strokeMiterLimit ( unsigned int miterLimit_ )
+{
+  _drawInfo->miterlimit = miterLimit_;
+}
+unsigned int Magick::Options::strokeMiterLimit ( void ) const
+{
+  return _drawInfo->miterlimit;
+}
+
 // Pattern image to use for stroked outlines
 void Magick::Options::strokePattern ( const MagickLib::Image *strokePattern_ )
 {
@@ -357,7 +552,7 @@ void Magick::Options::strokePattern ( const MagickLib::Image *strokePattern_ )
 	CloneImage( const_cast<MagickLib::Image*>(strokePattern_),
 		    0,
 		    0,
-		    static_cast<int>(true),
+		    MagickTrue,
 		    &exceptionInfo );
       throwException( exceptionInfo );
     }
@@ -367,11 +562,51 @@ const MagickLib::Image* Magick::Options::strokePattern ( void  ) const
   return _drawInfo->stroke_pattern;
 }
 
+// Stroke width for drawing lines, circles, ellipses, etc.
+void Magick::Options::strokeWidth ( double strokeWidth_ )
+{
+  _drawInfo->stroke_width = strokeWidth_;
+}
+double Magick::Options::strokeWidth ( void ) const
+{
+  return _drawInfo->stroke_width;
+}
+
+void Magick::Options::subImage ( unsigned int subImage_ )
+{
+  _imageInfo->subimage = subImage_;
+}
+unsigned int Magick::Options::subImage ( void ) const
+{
+  return _imageInfo->subimage;
+}
+
+void Magick::Options::subRange ( unsigned int subRange_ )
+{
+  _imageInfo->subrange = subRange_;
+}
+unsigned int Magick::Options::subRange ( void ) const
+{
+  return _imageInfo->subrange;
+}
+
+// Annotation text encoding (e.g. "UTF-16")
+void Magick::Options::textEncoding ( const std::string &encoding_ )
+{
+  CloneString(&_drawInfo->encoding, encoding_.c_str());
+}
+std::string Magick::Options::textEncoding ( void ) const
+{
+  if ( _drawInfo->encoding && *_drawInfo->encoding )
+    return std::string( _drawInfo->encoding );
+  
+  return std::string();
+}
 
 void Magick::Options::tileName ( const std::string &tileName_ )
 {
   if ( tileName_.length() == 0 )
-    LiberateMemory( reinterpret_cast<void**>(&_imageInfo->tile) );
+    _imageInfo->tile=(char *) RelinquishMagickMemory(_imageInfo->tile);
   else
     Magick::CloneString( &_imageInfo->tile, tileName_ );
 }
@@ -382,6 +617,16 @@ std::string Magick::Options::tileName ( void ) const
   return std::string();
 }
 
+// Image representation type
+void Magick::Options::type ( const Magick::ImageType type_ )
+{
+  _imageInfo->type = type_;
+}
+Magick::ImageType Magick::Options::type ( void ) const
+{
+  return _imageInfo->type;
+}
+
 // Origin of coordinate system to use when annotating with text or drawing
 void Magick::Options::transformOrigin ( double tx_, double ty_ )
 {
@@ -512,10 +757,19 @@ void Magick::Options::transformSkewY ( double skewy_ )
   _drawInfo->affine.ty=current.rx*affine.tx+current.sy*affine.ty+current.ty;
 }
 
+void Magick::Options::verbose ( bool verboseFlag_ )
+{
+  _imageInfo->verbose = (MagickBooleanType) verboseFlag_;
+}
+bool Magick::Options::verbose ( void ) const
+{
+  return static_cast<bool>(_imageInfo->verbose);
+}
+
 void Magick::Options::view ( const std::string &view_ )
 {
   if ( view_.length() == 0 )
-    LiberateMemory( reinterpret_cast<void**>(&_imageInfo->view) );
+    _imageInfo->view=(char *) RelinquishMagickMemory(_imageInfo->view);
   else
     Magick::CloneString( &_imageInfo->view, view_ );
 }
@@ -530,7 +784,7 @@ std::string Magick::Options::view ( void ) const
 void Magick::Options::x11Display ( const std::string &display_ )
 {
   if ( display_.length() == 0 )
-    LiberateMemory( reinterpret_cast<void**>(&_imageInfo->server_name) );
+    _imageInfo->server_name=(char *) RelinquishMagickMemory(_imageInfo->server_name);
   else
     Magick::CloneString( &_imageInfo->server_name, display_ );
 }
@@ -539,5 +793,24 @@ std::string Magick::Options::x11Display ( void ) const
   if ( _imageInfo->server_name )
     return std::string( _imageInfo->server_name );
 
-    return std::string();
+  return std::string();
+}
+
+//
+// Internal implementation methods.  Please do not use.
+//
+
+MagickLib::DrawInfo * Magick::Options::drawInfo( void )
+{
+  return _drawInfo;
+}
+
+MagickLib::ImageInfo * Magick::Options::imageInfo( void )
+{
+  return _imageInfo;
+}
+
+MagickLib::QuantizeInfo * Magick::Options::quantizeInfo( void )
+{
+  return _quantizeInfo;
 }
