@@ -10,39 +10,26 @@
 %                            F      A   A  X   X                              %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%                   Read/Write Group 3 Fax Image Format.                      %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,12 +39,23 @@
 /*
   Include declarations.
 */
-#include "studio.h"
+#include "magick/studio.h"
+#include "magick/blob.h"
+#include "magick/colorspace.h"
+#include "magick/error.h"
+#include "magick/compress.h"
+#include "magick/image.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/static.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteFAXImage(const ImageInfo *,Image *);
 
 /*
@@ -71,17 +69,15 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method IsFAX returns True if the image format type, identified by the
+%  IsFAX() returns MagickTrue if the image format type, identified by the
 %  magick string, is FAX.
 %
 %  The format of the IsFAX method is:
 %
-%      unsigned int IsFAX(const unsigned char *magick,const size_t length)
+%      MagickBooleanType IsFAX(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o status:  Method IsFAX returns True if the image format type is FAX.
-%
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
@@ -89,13 +85,13 @@ static unsigned int
 %
 %
 */
-static unsigned int IsFAX(const unsigned char *magick,const size_t length)
+static MagickBooleanType IsFAX(const unsigned char *magick,const size_t length)
 {
   if (length < 4)
-    return(False);
+    return(MagickFalse);
   if (LocaleNCompare((char *) magick,"DFAX",4) == 0)
-    return(True);
-  return(False);
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -109,7 +105,7 @@ static unsigned int IsFAX(const unsigned char *magick,const size_t length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadFAXImage reads a Group 3 FAX image file and returns it.  It
+%  ReadFAXImage() reads a Group 3 FAX image file and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -119,11 +115,7 @@ static unsigned int IsFAX(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadFAXImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -134,7 +126,7 @@ static Image *ReadFAXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image
     *image;
 
-  unsigned int
+  MagickBooleanType
     status;
 
   /*
@@ -142,12 +134,17 @@ static Image *ReadFAXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
+  if (status == MagickFalse)
+    {
+      DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Initialize image structure.
   */
@@ -157,8 +154,8 @@ static Image *ReadFAXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if (image->rows == 0)
     image->rows=3508;
   image->depth=8;
-  if (!AllocateImageColormap(image,2))
-    ThrowReaderException(ResourceLimitError,"Memory allocation failed",image);
+  if (AllocateImageColormap(image,2) == MagickFalse)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   /*
     Monochrome colormap.
   */
@@ -168,18 +165,19 @@ static Image *ReadFAXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   image->colormap[1].red=0;
   image->colormap[1].green=0;
   image->colormap[1].blue=0;
-  if (image_info->ping)
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-      return(image);
+      return(GetFirstImageInList(image));
     }
   status=HuffmanDecodeImage(image);
-  if (status == False)
-    ThrowReaderException(CorruptImageError,"Unable to read image data",image);
-  if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+  if (status == MagickFalse)
+    ThrowReaderException(CorruptImageError,"UnableToReadImageData");
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -193,7 +191,7 @@ static Image *ReadFAXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterFAXImage adds attributes for the FAX image format to
+%  RegisterFAXImage() adds attributes for the FAX image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -210,18 +208,28 @@ ModuleExport void RegisterFAXImage(void)
   MagickInfo
     *entry;
 
+  static const char
+    *Note=
+    {
+      "See TIFF format.  Note that FAX machines use non-square pixels which\n"
+      "are 1.5 times wider than they are tall but computer displays use\n"
+      "square pixels. FAX images may appear to be narrow unless they are\n"
+      "explicitly resized using a resize specification of \"150x100%\"."
+    };
+
   entry=SetMagickInfo("FAX");
-  entry->decoder=ReadFAXImage;
-  entry->encoder=WriteFAXImage;
-  entry->magick=IsFAX;
+  entry->decoder=(DecoderHandler *) ReadFAXImage;
+  entry->encoder=(EncoderHandler *) WriteFAXImage;
+  entry->magick=(MagickHandler *) IsFAX;
   entry->description=AcquireString("Group 3 FAX");
+  entry->note=AcquireString(Note);
   entry->module=AcquireString("FAX");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("G3");
-  entry->decoder=ReadFAXImage;
-  entry->encoder=WriteFAXImage;
-  entry->magick=IsFAX;
-  entry->adjoin=False;
+  entry->decoder=(DecoderHandler *) ReadFAXImage;
+  entry->encoder=(EncoderHandler *) WriteFAXImage;
+  entry->magick=(MagickHandler *) IsFAX;
+  entry->adjoin=MagickFalse;
   entry->description=AcquireString("Group 3 FAX");
   entry->module=AcquireString("FAX");
   (void) RegisterMagickInfo(entry);
@@ -238,7 +246,7 @@ ModuleExport void RegisterFAXImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterFAXImage removes format registrations made by the
+%  UnregisterFAXImage() removes format registrations made by the
 %  FAX module from the list of supported formats.
 %
 %  The format of the UnregisterFAXImage method is:
@@ -263,34 +271,32 @@ ModuleExport void UnregisterFAXImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Procedure WriteFAXImage writes an image to a file in 1 dimensional Huffman
+%  Procedure WriteFAXImage() writes an image to a file in 1 dimensional Huffman
 %  encoded format.
 %
 %  The format of the WriteFAXImage method is:
 %
-%      unsigned int WriteFAXImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteFAXImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteFAXImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
+%    o image_info: The image info.
 %
-%    o image_info: Specifies a pointer to an ImageInfo structure.
-%
-%    o image:  A pointer to a Image structure.
+%    o image:  The image.
 %
 %
 */
-static unsigned int WriteFAXImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteFAXImage(const ImageInfo *image_info,Image *image)
 {
   ImageInfo
-    *clone_info;
+    *write_info;
 
-  unsigned int
-    scene,
+  MagickBooleanType
     status;
 
+  MagickOffsetType
+    scene;
+
   /*
     Open output image file.
   */
@@ -298,28 +304,34 @@ static unsigned int WriteFAXImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
-  clone_info=CloneImageInfo(image_info);
-  (void) strcpy(clone_info->magick,"FAX");
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  if (status == MagickFalse)
+    return(status);
+  write_info=CloneImageInfo(image_info);
+  (void) strcpy(write_info->magick,"FAX");
   scene=0;
   do
   {
     /*
       Convert MIFF to monochrome.
     */
-    (void) TransformRGBImage(image,RGBColorspace);
-    status=HuffmanEncodeImage(clone_info,image);
+    (void) SetImageColorspace(image,RGBColorspace);
+    status=HuffmanEncodeImage(write_info,image);
     if (image->next == (Image *) NULL)
       break;
-    image=GetNextImage(image);
-    MagickMonitor(SaveImagesText,scene++,GetImageListSize(image));
-  } while (clone_info->adjoin);
-  DestroyImageInfo(clone_info);
-  if (image_info->adjoin)
-    while (image->previous != (Image *) NULL)
-      image=image->previous;
+    image=SyncNextImageInList(image);
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        status=image->progress_monitor(SaveImagesTag,scene,
+          GetImageListLength(image),image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+    scene++;
+  } while (write_info->adjoin != MagickFalse);
+  DestroyImageInfo(write_info);
   CloseBlob(image);
   return(status);
 }
