@@ -10,39 +10,26 @@
 %                         UUU     Y      V      Y                             %
 %                                                                             %
 %                                                                             %
-%                    Read/Write ImageMagick Image Format.                     %
-%                                                                             %
+%            Read/Write 16bit/pixel Interleaved YUV Image Format.             %
 %                                                                             %
 %                              Software Design                                %
 %                                John Cristy                                  %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -52,12 +39,23 @@
 /*
   Include declarations.
 */
-#include "studio.h"
+#include "magick/studio.h"
+#include "magick/blob.h"
+#include "magick/color.h"
+#include "magick/colorspace.h"
+#include "magick/error.h"
+#include "magick/image.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/static.h"
+#include "magick/string_.h"
 
 /*
   Forward declarations.
 */
-static unsigned int
+static MagickBooleanType
   WriteUYVYImage(const ImageInfo *,Image *);
 
 /*
@@ -71,7 +69,7 @@ static unsigned int
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadUYVYImage reads an image in the UYVY format and returns it.  It
+%  ReadUYVYImage() reads an image in the UYVY format and returns it.  It
 %  allocates the memory necessary for the new Image structure and returns a
 %  pointer to the new image.
 %
@@ -82,11 +80,7 @@ static unsigned int
 %
 %  A description of each parameter follows:
 %
-%    o image:  Method ReadUYVYImage returns a pointer to the image after
-%      reading.  A null image is returned if there is a memory shortage or
-%      if the image cannot be read.
-%
-%    o image_info: Specifies a pointer to an ImageInfo structure.
+%    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure.
 %
@@ -101,6 +95,9 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   register long
     x;
 
@@ -116,30 +113,29 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
     y1,
     y2;
 
-  unsigned int
-    status;
-
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,"Must specify image size",image);
-  (void) strncpy(image->filename,image_info->filename,MaxTextExtent-1);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image)
+    ThrowReaderException(OptionError,"MustSpecifyImageSize");
+  (void) CopyMagickString(image->filename,image_info->filename,MaxTextExtent);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
+  if (status == MagickFalse)
+    return((Image *) NULL);
   for (i=0; i < image->offset; i++)
     (void) ReadBlobByte(image);
   image->depth=8;
-  if (image_info->ping)
+  if (image_info->ping != MagickFalse)
     {
       CloseBlob(image);
-      return(image);
+      return(GetFirstImageInList(image));
     }
   /*
     Accumulate UYVY, then unpack into two pixels.
@@ -151,29 +147,37 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
       break;
     for (x=0; x < (long) (image->columns >> 1); x++)
     {
-      u=ReadBlobByte(image);
-      y1=ReadBlobByte(image);
-      v=ReadBlobByte(image);
-      y2=ReadBlobByte(image);
-      q->red=Upscale(y1);
-      q->green=Upscale(u);
-      q->blue=Upscale(v);
+      u=(unsigned char) ReadBlobByte(image);
+      y1=(unsigned char) ReadBlobByte(image);
+      v=(unsigned char) ReadBlobByte(image);
+      y2=(unsigned char) ReadBlobByte(image);
+      q->red=ScaleCharToQuantum(y1);
+      q->green=ScaleCharToQuantum(u);
+      q->blue=ScaleCharToQuantum(v);
       q++;
-      q->red=Upscale(y2);
-      q->green=Upscale(u);
-      q->blue=Upscale(v);
+      q->red=ScaleCharToQuantum(y2);
+      q->green=ScaleCharToQuantum(u);
+      q->blue=ScaleCharToQuantum(v);
       q++;
     }
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
-  (void) TransformRGBImage(image,YCbCrColorspace);
-  if (EOFBlob(image))
-    ThrowReaderException(CorruptImageError,"Unexpected end-of-file",image);
+  image->colorspace=YCbCrColorspace;
+  (void) SetImageColorspace(image,RGBColorspace);
+  if (EOFBlob(image) != MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   CloseBlob(image);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -187,7 +191,7 @@ static Image *ReadUYVYImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method RegisterUYVYImage adds attributes for the UYVY image format to
+%  RegisterUYVYImage() adds attributes for the UYVY image format to
 %  the list of supported formats.  The attributes include the image format
 %  tag, a method to read and/or write the format, whether the format
 %  supports the saving of more than one frame to the same file or blob,
@@ -205,18 +209,20 @@ ModuleExport void RegisterUYVYImage(void)
     *entry;
 
   entry=SetMagickInfo("PAL");
-  entry->decoder=ReadUYVYImage;
-  entry->encoder=WriteUYVYImage;
-  entry->adjoin=False;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadUYVYImage;
+  entry->encoder=(EncoderHandler *) WriteUYVYImage;
+  entry->adjoin=MagickFalse;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("16bit/pixel interleaved YUV");
   entry->module=AcquireString("UYVY");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("UYVY");
-  entry->decoder=ReadUYVYImage;
-  entry->encoder=WriteUYVYImage;
-  entry->adjoin=False;
-  entry->raw=True;
+  entry->decoder=(DecoderHandler *) ReadUYVYImage;
+  entry->encoder=(EncoderHandler *) WriteUYVYImage;
+  entry->adjoin=MagickFalse;
+  entry->raw=MagickTrue;
+  entry->endian_support=MagickTrue;
   entry->description=AcquireString("16bit/pixel interleaved YUV");
   entry->module=AcquireString("UYVY");
   (void) RegisterMagickInfo(entry);
@@ -233,7 +239,7 @@ ModuleExport void RegisterUYVYImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method UnregisterUYVYImage removes format registrations made by the
+%  UnregisterUYVYImage() removes format registrations made by the
 %  UYVY module from the list of supported formats.
 %
 %  The format of the UnregisterUYVYImage method is:
@@ -258,44 +264,45 @@ ModuleExport void UnregisterUYVYImage(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method WriteUYVYImage writes an image to a file in the digital UYVY
+%  WriteUYVYImage() writes an image to a file in the digital UYVY
 %  format.  This format, used by AccomWSD, is not dramatically higher quality
 %  than the 12bit/pixel YUV format, but has better locality.
 %
 %  The format of the WriteUYVYImage method is:
 %
-%      unsigned int WriteUYVYImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteUYVYImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  A description of each parameter follows.
 %
-%    o status: Method WriteUYVYImage return True if the image is written.
-%      False is returned is there is a memory shortage or if the image file
-%      fails to write.
+%    o image_info: The image info.
 %
-%    o image_info: Specifies a pointer to an ImageInfo structure.
-%
-%    o image:  A pointer to a Image structure.
+%    o image:  The image.
 %      Implicit assumption: number of columns is even.
 %
 */
-static unsigned int WriteUYVYImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteUYVYImage(const ImageInfo *image_info,
+  Image *image)
 {
+  MagickPixelPacket
+    pixel;
+
+  Image
+    *uyvy_image;
+
   long
     y;
 
+  MagickBooleanType
+    full,
+    status;
+
   register const PixelPacket
     *p;
 
   register long
     x;
 
-  unsigned int
-    full,
-    status,
-    u,
-    v,
-    y1;
-
   /*
     Open output image file.
   */
@@ -303,50 +310,52 @@ static unsigned int WriteUYVYImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
-  status=OpenBlob(image_info,image,WriteBinaryType,&image->exception);
-  if (status == False)
-    ThrowWriterException(FileOpenError,"Unable to open file",image);
-  (void) TransformRGBImage(image,RGBColorspace);
-  /*
-    Convert to YUV, at full resolution.
-  */
-  (void) RGBTransformImage(image,YCbCrColorspace);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),image->filename);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  if (status == MagickFalse)
+    return(status);
   /*
     Accumulate two pixels, then output.
   */
-  full=False;
-  u=0;
-  v=0;
-  y1=0;
-  full=False;
+  uyvy_image=CloneImage(image,0,0,MagickTrue,&image->exception);
+  if (uyvy_image == (Image *) NULL)
+    ThrowWriterException(ResourceLimitError,image->exception.reason);
+  (void) SetImageColorspace(uyvy_image,YCbCrColorspace);
+  full=MagickFalse;
+  (void) ResetMagickMemory(&pixel,0,sizeof(MagickPixelPacket));
   for (y=0; y < (long) image->rows; y++)
   {
-    p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
+    p=AcquireImagePixels(uyvy_image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
     for (x=0; x < (long) image->columns; x++)
     {
-      if (full)
+      if (full != MagickFalse)
         {
-          (void) WriteBlobByte(image,Downscale((u+p->green) >> 1));
-          (void) WriteBlobByte(image,Downscale(y1));
-          (void) WriteBlobByte(image,Downscale((v+p->blue) >> 1));
-          (void) WriteBlobByte(image,Downscale(p->red));
-          full=False;
-        }
-      else
-        {
-          y1=p->red;
-          u=p->green;
-          v=p->blue;
-          full=True;
+          pixel.green=(pixel.green+p->green)/2;
+          pixel.blue=(pixel.blue+p->blue)/2;
+          (void) WriteBlobByte(image,ScaleQuantumToChar(pixel.green));
+          (void) WriteBlobByte(image,ScaleQuantumToChar(pixel.red));
+          (void) WriteBlobByte(image,ScaleQuantumToChar(pixel.blue));
+          (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
         }
+      pixel.red=(double) p->red;
+      pixel.green=(double) p->green;
+      pixel.blue=(double) p->blue;
+      full=(MagickBooleanType) (full == MagickFalse);
       p++;
     }
-    if (QuantumTick(y,image->rows))
-      MagickMonitor(SaveImageText,y,image->rows);
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(SaveImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
-  (void) TransformRGBImage(image,YCbCrColorspace);
+  DestroyImage(uyvy_image);
   CloseBlob(image);
-  return(True);
+  return(MagickTrue);
 }
