@@ -12,37 +12,24 @@
 %                                                                             %
 %                   Macintosh Utility Methods for ImageMagick                 %
 %                                                                             %
-%                                                                             %
 %                               Software Design                               %
 %                                 John Cristy                                 %
 %                                September 1996                               %
 %                                                                             %
 %                                                                             %
-%  Copyright (C) 2002 ImageMagick Studio, a non-profit organization dedicated %
-%  to making software imaging solutions freely available.                     %
+%  Copyright 1999-2004 ImageMagick Studio LLC, a non-profit organization      %
+%  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
-%  Permission is hereby granted, free of charge, to any person obtaining a    %
-%  copy of this software and associated documentation files ("ImageMagick"),  %
-%  to deal in ImageMagick without restriction, including without limitation   %
-%  the rights to use, copy, modify, merge, publish, distribute, sublicense,   %
-%  and/or sell copies of ImageMagick, and to permit persons to whom the       %
-%  ImageMagick is furnished to do so, subject to the following conditions:    %
+%  You may not use this file except in compliance with the License.  You may  %
+%  obtain a copy of the License at                                            %
 %                                                                             %
-%  The above copyright notice and this permission notice shall be included in %
-%  all copies or substantial portions of ImageMagick.                         %
+%    http://www.imagemagick.org/www/Copyright.html                            %
 %                                                                             %
-%  The software is provided "as is", without warranty of any kind, express or %
-%  implied, including but not limited to the warranties of merchantability,   %
-%  fitness for a particular purpose and noninfringement.  In no event shall   %
-%  ImageMagick Studio be liable for any claim, damages or other liability,    %
-%  whether in an action of contract, tort or otherwise, arising from, out of  %
-%  or in connection with ImageMagick or the use or other dealings in          %
-%  ImageMagick.                                                               %
-%                                                                             %
-%  Except as contained in this notice, the name of the ImageMagick Studio     %
-%  shall not be used in advertising or otherwise to promote the sale, use or  %
-%  other dealings in ImageMagick without prior written authorization from the %
-%  ImageMagick Studio.                                                        %
+%  Unless required by applicable law or agreed to in writing, software        %
+%  distributed under the License is distributed on an "AS IS" BASIS,          %
+%  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
+%  See the License for the specific language governing permissions and        %
+%  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
@@ -75,8 +62,16 @@
 #include <Gestalt.h>
 #include <TextUtils.h>
 #define ColorInfo  KolorInfo
-#include "studio.h"
-#include "mac.h"
+#include "magick/studio.h"
+#include "magick/blob.h"
+#include "magick/client.h"
+#include "magick/error.h"
+#include "magick/list.h"
+#include "magick/magick.h"
+#include "magick/monitor.h"
+#include "magick/string_.h"
+#include "magick/utility.h"
+#include "magick/mac.h"
 
 /*
   Global declaractions.
@@ -104,6 +99,17 @@ static pascal void
     Rect *,short),
   TextMethod(short,Ptr,Point,Point);
 
+/*
+  Static declarations
+ */
+#if defined(DISABLE_SIOUX)
+static MACEventHookPtr
+  event_hook = nil;
+
+static MACErrorHookPtr
+  error_hook = nil;
+#endif
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -115,7 +121,7 @@ static pascal void
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method BottleneckTest intercepts any compressed images.
+%  BottleneckTest() intercepts any compressed images.
 %
 %  The format of the BottleneckTest method is:
 %
@@ -198,6 +204,7 @@ static pascal void TextMethod(short byteCount,Ptr textBuf,Point numer,
 #pragma unused (byteCount,textBuf,numer,denom)
 }
 
+#if !defined(DISABLE_QUICKTIME)
 static short BottleneckTest(PicHandle picture,CodecType *codec,int *depth,
   short *colormap_id)
 {
@@ -217,7 +224,7 @@ static short BottleneckTest(PicHandle picture,CodecType *codec,int *depth,
   if (status != noErr)
     {
       ParamText("\pQuickTime not installed.  Please install, then try again.",
-        "\p", "\p", "\p" );
+        "\p","\p","\p");
       Alert(128,nil);
       return(-1);
     }
@@ -225,16 +232,16 @@ static short BottleneckTest(PicHandle picture,CodecType *codec,int *depth,
     Define our own bottlenecks to do nothing.
   */
   SetStdCProcs(&bottlenecks);
-  bottlenecks.textProc=NewQDTextProc(&TextMethod);
-  bottlenecks.lineProc=NewQDLineProc(&LineMethod);
-  bottlenecks.rectProc=NewQDRectProc(&RectMethod);
-  bottlenecks.rRectProc=NewQDRRectProc(&RRectMethod);
-  bottlenecks.ovalProc=NewQDOvalProc(&OvalMethod);
-  bottlenecks.arcProc=NewQDArcProc(&ArcMethod);
-  bottlenecks.polyProc=NewQDPolyProc(&PolyMethod);
-  bottlenecks.rgnProc=NewQDRgnProc(&RegionMethod);
-  bottlenecks.bitsProc=NewQDBitsProc(&BitsMethod);
-  bottlenecks.newProc1=(RoutineDescriptor *) NewStdPixProc(&StandardPixmap);
+  bottlenecks.textProc=NewQDTextUPP(&TextMethod);
+  bottlenecks.lineProc=NewQDLineUPP(&LineMethod);
+  bottlenecks.rectProc=NewQDRectUPP(&RectMethod);
+  bottlenecks.rRectProc=NewQDRRectUPP(&RRectMethod);
+  bottlenecks.ovalProc=NewQDOvalUPP(&OvalMethod);
+  bottlenecks.arcProc=NewQDArcUPP(&ArcMethod);
+  bottlenecks.polyProc=NewQDPolyUPP(&PolyMethod);
+  bottlenecks.rgnProc=NewQDRgnUPP(&RegionMethod);
+  bottlenecks.bitsProc=NewQDBitsUPP(&BitsMethod);
+  bottlenecks.newProc1=(UniversalProcPtr) NewStdPixUPP(&StandardPixmap);
   /*
     Install our custom bottlenecks to intercept any compressed images.
   */
@@ -254,19 +261,21 @@ static short BottleneckTest(PicHandle picture,CodecType *codec,int *depth,
       *depth=(**image_description).depth;
       *colormap_id=(**image_description).clutID;
     }
-  DisposeRoutineDescriptor(bottlenecks.textProc);
-  DisposeRoutineDescriptor(bottlenecks.lineProc);
-  DisposeRoutineDescriptor(bottlenecks.rectProc);
-  DisposeRoutineDescriptor(bottlenecks.rRectProc);
-  DisposeRoutineDescriptor(bottlenecks.ovalProc);
-  DisposeRoutineDescriptor(bottlenecks.arcProc);
-  DisposeRoutineDescriptor(bottlenecks.polyProc);
-  DisposeRoutineDescriptor(bottlenecks.rgnProc);
-  DisposeRoutineDescriptor(bottlenecks.bitsProc);
-  DisposeRoutineDescriptor(bottlenecks.newProc1);
+  DisposeQDTextUPP(bottlenecks.textProc);
+  DisposeQDLineUPP(bottlenecks.lineProc);
+  DisposeQDRectUPP(bottlenecks.rectProc);
+  DisposeQDRRectUPP(bottlenecks.rRectProc);
+  DisposeQDOvalUPP(bottlenecks.ovalProc);
+  DisposeQDArcUPP(bottlenecks.arcProc);
+  DisposeQDPolyUPP(bottlenecks.polyProc);
+  DisposeQDRgnUPP(bottlenecks.rgnProc);
+  DisposeQDBitsUPP(bottlenecks.bitsProc);
+  DisposeStdPixUPP(bottlenecks.newProc1);
   return(0);
 }
+#endif
 
+#if !defined(_POSIX_VERSION)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -278,8 +287,7 @@ static short BottleneckTest(PicHandle picture,CodecType *codec,int *depth,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method closedir closes the named directory stream and frees the DIR
-%  structure.
+%  closedir() closes the named directory stream and frees the DIR structure.
 %
 %  The format of the closedir method is:
 %
@@ -293,9 +301,12 @@ static short BottleneckTest(PicHandle picture,CodecType *codec,int *depth,
 */
 MagickExport void closedir(DIR *entry)
 {
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(entry != (DIR *) NULL);
-  LiberateMemory((void **) &entry);
+  RelinquishMagickMemory(entry);
 }
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -308,7 +319,7 @@ MagickExport void closedir(DIR *entry)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method Exit exits the process.
+%  Exit() exits the process.
 %
 %  The format of the Exit method is:
 %
@@ -323,7 +334,9 @@ MagickExport void closedir(DIR *entry)
 */
 MagickExport int Exit(int status)
 {
-  (void) fprintf(stderr,"Select File->Quit to exit.\n");
+#if !defined(DISABLE_SIOUX)
+  (void) fprintf(stdout,"Select File->Quit to exit.\n");
+#endif
   exit(status);
   return(0);
 }
@@ -339,7 +352,7 @@ MagickExport int Exit(int status)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method FilenameToFSSpec sets the file type of an image.
+%  FilenameToFSSpec() sets the file type of an image.
 %
 %  The format of the FilenameToFSSpec method is:
 %
@@ -359,8 +372,7 @@ MagickExport void pascal FilenameToFSSpec(const char *filename,FSSpec *fsspec)
     name;
 
   assert(filename != (char *) NULL);
-  (void) strncpy((char *) name,filename,254);
-  c2pstr((char *) name);
+  c2pstrcpy(name, filename);
   FSMakeFSSpec(0,0,name,fsspec);
 }
 
@@ -375,8 +387,8 @@ MagickExport void pascal FilenameToFSSpec(const char *filename,FSSpec *fsspec)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method MACIsMagickConflict returns true if the image format conflicts with
-%  a logical drive (.e.g. X:).
+%  MACIsMagickConflict() returns true if the image format conflicts with a
+%  logical drive (.e.g. X:).
 %
 %  Contributed by Mark Gavin of Digital Applications, Inc.
 %
@@ -386,9 +398,6 @@ MagickExport void pascal FilenameToFSSpec(const char *filename,FSSpec *fsspec)
 %
 %  A description of each parameter follows:
 %
-%    o status: Method MACIsMagickConflict returns true if the image format
-%      conflicts with a logical drive.
-%
 %    o magick: Specifies the image format.
 %
 %
@@ -402,7 +411,7 @@ static OSErr HGetVInfo(short volume_index,StringPtr volume_name,short *volume,
 
   OSErr
     result;
-  
+
   unsigned long
     blocksize;
 
@@ -427,8 +436,8 @@ static OSErr HGetVInfo(short volume_index,StringPtr volume_name,short *volume,
   *total_bytes=allocation_blocks*blocksize;
   return(result);
 }
- 
-MagickExport unsigned int MACIsMagickConflict(const char *magick)
+
+MagickExport MagickBooleanType MACIsMagickConflict(const char *magick)
 {
   unsigned long
     free_bytes,
@@ -438,26 +447,20 @@ MagickExport unsigned int MACIsMagickConflict(const char *magick)
     status;
 
   short
-    volume,
-    index;
+    volume;
 
   Str255
     volume_name;
 
-  StringPtr
-    p;
-  
   assert(magick != (char *) NULL);
-  p=(StringPtr) &volume_name;
-  for (index=(-1); ; index--)
-  {
-    status=HGetVInfo(index,p,&volume,&free_bytes,&number_bytes);
-    if (status)
-      return(False);
-    if (LocaleCompare(p2cstr(p),magick) == 0)
-      return(True);
-  }
-  return(False);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),magick);
+  (void) strcpy((char *) volume_name,magick);
+  c2pstr((char *) volume_name);
+  if (volume_name[volume_name[0]] != ':')
+    volume_name[++volume_name[0]]=':';
+  status=HGetVInfo(-1,volume_name,&volume,&free_bytes,&number_bytes);
+  return(status != 0 ? MagickFalse : MagickTrue);
 }
 
 /*
@@ -471,8 +474,7 @@ MagickExport unsigned int MACIsMagickConflict(const char *magick)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method MACErrorHandler displays an error reason and then terminates
-%  the program.
+%  MACErrorHandler() displays an error reason and then terminates the program.
 %
 %  The format of the MACErrorHandler method is:
 %
@@ -497,29 +499,83 @@ MagickExport void MACErrorHandler(const ExceptionType error,const char *reason,
     buffer[3*MaxTextExtent];
 
   if (reason == (char *) NULL)
+    return;
+  if (description == (char *) NULL)
+    (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s.\n",GetClientName(),
+      reason);
+  else
+    (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s (%s).\n",
+      GetClientName(),reason,description);
+#if defined(DISABLE_SIOUX)
+  if(error_hook != (MACErrorHookPtr) NULL)
+    error_hook(error,buffer);
+  else
     {
       DestroyMagick();
-      Exit(0);
+      Exit(error);
     }
-  if ((description != (char *) NULL) && errno)
-    FormatString(buffer,"%.1024s: %.1024s (%.1024s) [%.1024s].\n",
-      SetClientName((char *) NULL),reason,description,strerror(errno));
-  else
-    if (description != (char *) NULL)
-      FormatString(buffer,"%.1024s: %.1024s (%.1024s).\n",
-        SetClientName((char *) NULL),reason,description);
-    else
-      if (errno)
-        FormatString(buffer,"%.1024s: %.1024s [%.1024s].\n",
-          SetClientName((char *) NULL),reason,strerror(errno));
-      else
-        FormatString(buffer,"%.1024s: %.1024s.\n",SetClientName((char *) NULL),
-          reason);
+#else
   puts(buffer);
   DestroyMagick();
-  Exit(0);
+  Exit(error);
+#endif
 }
 
+#if defined(DISABLE_SIOUX)
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   M A C F a t a l E r r o r H a n d l e r                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MACFatalErrorHandler() displays an error reason and then terminates the
+%  program.
+%
+%  The format of the MACFatalErrorHandler method is:
+%
+%      void MACFatalErrorHandler(const ExceptionType severity,
+%        const char *reason,const char *description)
+%
+%  A description of each parameter follows:
+%
+%    o severity: Specifies the numeric error category.
+%
+%    o reason: Specifies the reason to display before terminating the
+%      program.
+%
+%    o description: Specifies any description to the reason.
+%
+%
+*/
+static void MACFatalErrorHandler(const ExceptionType severity,
+  const char *reason,const char *description)
+{
+  char
+    buffer[3*MaxTextExtent];
+
+  if (reason == (char *) NULL)
+    return;
+  if (description == (char *) NULL)
+    (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s.\n",GetClientName(),
+      reason);
+  else
+    (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s (%s).\n",
+      GetClientName(),reason,description);
+  if(error_hook != (MACErrorHookPtr) NULL)
+    error_hook(severity, buffer);
+  else
+    {
+      DestroyMagick();
+      Exit(severity);
+    }
+}
+#endif
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -531,7 +587,7 @@ MagickExport void MACErrorHandler(const ExceptionType error,const char *reason,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method MacGSExecuteCommand executes the Ghostscript command.
+%  MacGSExecuteCommand() executes the Ghostscript command.
 %
 %
 */
@@ -590,7 +646,7 @@ static OSErr MacGSExecuteCommand(const char *command,long length)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method MacGSLaunchApplicationCore launches the Ghostscript command.
+%  MacGSLaunchApplicationCore() launches the Ghostscript command.
 %
 %
 */
@@ -628,7 +684,7 @@ static OSErr MacGSLaunchApplicationCore(long flags)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method MacGSLaunchApplication launches the Ghostscript command.
+%  MacGSLaunchApplication() launches the Ghostscript command.
 %
 %
 */
@@ -648,8 +704,7 @@ static OSErr MacGSLaunchApplication(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method MacGSLaunchApplicationToFront moves the Ghostscript window to the
-%  front.
+%  MacGSLaunchApplicationToFront() moves the Ghostscript window to the front.
 %
 %
 */
@@ -669,7 +724,7 @@ static OSErr MacGSLaunchApplicationToFront(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method MacGSQuitApplication quits the Ghostscript application.
+%  MacGSQuitApplication() quits the Ghostscript application.
 %
 %
 */
@@ -715,7 +770,7 @@ static void MacGSQuitApplication(void)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method MacGSSetWorkingFolder set the Ghostscript working folder.
+%  MacGSSetWorkingFolder() set the Ghostscript working folder.
 %
 %
 */
@@ -770,6 +825,70 @@ static OSErr MacGSSetWorkingFolder(char *directory)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M A C S e t E r r o r H o o k                                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%   MACSetErrorHook sets a callback function which is called if any error
+%   occurs within ImageMagick.
+%
+%  The format of the MACSetErrorHook method is:
+%
+%      int MACSetErrorHook(MACErrorHookPtr hook)
+%
+%  A description of each parameter follows:
+%
+%    o hook: This function pointer is the callback function.
+%
+%
+*/
+MagickExport void MACSetErrorHook(MACErrorHookPtr hook)
+{
+  /*
+    We forget any previously set error hook.
+  */
+  error_hook=hook;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   M A C S e t E v e n t H o o k                                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%   MACSetEventHook sets a callback function which is called every time
+%   ImageMagick likes to release the processor.
+%
+%  The format of the MACSetEventHook method is:
+%
+%      int MACSetEventHook(MACEventHookPtr hook)
+%
+%  A description of each parameter follows:
+%
+%    o hook: This function pointer is the callback function.
+%
+%
+*/
+MagickExport void MACSetEventHook(MACEventHookPtr hook)
+{
+  /*
+    We forget any previously set event hook.
+   */
+  event_hook=hook;
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   M A C S y s t e m C o m m a n d                                           %
 %                                                                             %
 %                                                                             %
@@ -810,7 +929,7 @@ MagickExport int MACSystemCommand(const char * command)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method MACWarningHandler displays a warning reason.
+%  MACWarningHandler() displays a warning reason.
 %
 %  The format of the MACWarningHandler method is:
 %
@@ -837,14 +956,21 @@ MagickExport void MACWarningHandler(const ExceptionType warning,
   if (reason == (char *) NULL)
     return;
   if (description == (char *) NULL)
-    FormatString(buffer,"%.1024s: %.1024s.\n",
-      SetClientName((char *) NULL),reason);
+    (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s.\n",GetClientName(),
+      reason);
   else
-    FormatString(buffer,"%.1024s: %.1024s (%.1024s).\n",
-      SetClientName((char *) NULL),reason,description);
+    (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s (%s).\n",
+      GetClientName(),reason,description);
+#if defined(DISABLE_SIOUX)
+  if(error_hook != (MACErrorHookPtr) NULL)
+    error_hook(warning, buffer);
+#else
+  (void)warning;
   puts(buffer);
+#endif
 }
 
+#if !defined(_POSIX_VERSION)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -856,8 +982,8 @@ MagickExport void MACWarningHandler(const ExceptionType warning,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method opendir opens the directory named by filename and associates a
-%  directory stream with it.
+%  opendir() opens the directory named by filename and associates a directory
+%  stream with it.
 %
 %  The format of the opendir method is:
 %
@@ -871,8 +997,7 @@ MagickExport void MACWarningHandler(const ExceptionType warning,
 */
 MagickExport DIR *opendir(const char *path)
 {
-  char
-    pathname[MaxTextExtent];
+  Str255 pathname;
 
   CInfoPBRec
     search_info;
@@ -886,8 +1011,10 @@ MagickExport DIR *opendir(const char *path)
   search_info.hFileInfo.ioNamePtr=0;
   if ((path != (char *) NULL) || (*path != '\0'))
     if ((path[0] != '.') || (path[1] != '\0'))
-      search_info.hFileInfo.ioNamePtr=
-        c2pstr(strncpy(pathname,path,MaxTextExtent-1));
+      {
+        c2pstrcpy(pathname,path);
+        search_info.hFileInfo.ioNamePtr=pathname;
+      }
   search_info.hFileInfo.ioCompletion=0;
   search_info.hFileInfo.ioVRefNum=0;
   search_info.hFileInfo.ioFDirIndex=0;
@@ -898,7 +1025,7 @@ MagickExport DIR *opendir(const char *path)
       errno=error;
       return((DIR *) NULL);
     }
-  entry=(DIR *) AcquireMemory(sizeof(DIR));
+  entry=(DIR *) AcquireMagickMemory(sizeof(DIR));
   if (entry == (DIR *) NULL)
     return((DIR *) NULL);
   entry->d_VRefNum=search_info.hFileInfo.ioVRefNum;
@@ -906,6 +1033,7 @@ MagickExport DIR *opendir(const char *path)
   entry->d_index=1;
   return(entry);
 }
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -918,7 +1046,7 @@ MagickExport DIR *opendir(const char *path)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ProcessPendingEvents processes any pending events.  This prevents
+%  ProcessPendingEvents() processes any pending events.  This prevents
 %  ImageMagick from monopolizing the processor.
 %
 %  The format of the ProcessPendingEvents method is:
@@ -933,6 +1061,10 @@ MagickExport DIR *opendir(const char *path)
 */
 MagickExport void ProcessPendingEvents(const char *text)
 {
+#if defined(DISABLE_SIOUX)
+  if (event_hook != (MACEventHookPtr) NULL)
+    event_hook(text);
+#else
   static const char
     *mark = (char *) NULL;
 
@@ -946,8 +1078,10 @@ MagickExport void ProcessPendingEvents(const char *text)
       (void) puts(text);
       mark=text;
     }
+#endif
 }
 
+#if !defined(_POSIX_VERSION)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -959,9 +1093,8 @@ MagickExport void ProcessPendingEvents(const char *text)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method readdir returns a pointer to a structure representing the
-%  directory entry at the current position in the directory stream to
-%  which entry refers.
+%  readdir() returns a pointer to a structure representing the directory entry
+%  at the current position in the directory stream to which entry refers.
 %
 %  The format of the readdir
 %
@@ -1001,10 +1134,11 @@ MagickExport struct dirent *readdir(DIR *entry)
       return((struct dirent *) NULL);
     }
   entry->d_index++;
-  (void) strncpy(dir_entry.d_name,p2cstr(search_info.hFileInfo.ioNamePtr),254);
+  p2cstrcpy(dir_entry.d_name,search_info.hFileInfo.ioNamePtr);
   dir_entry.d_namlen=strlen(dir_entry.d_name);
   return(&dir_entry);
 }
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1017,10 +1151,9 @@ MagickExport struct dirent *readdir(DIR *entry)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method ReadPICTImage reads an Apple Macintosh QuickDraw/PICT image
-%  file using MacOS QuickDraw methods and returns it.  It allocates the
-%  memory necessary for the new Image structure and returns a pointer to
-%  the new image.
+%  ReadPICTImage() reads an Apple Macintosh QuickDraw/PICT image file using
+%  MacOS QuickDraw methods and returns it.  It allocates the memory necessary
+%  for the new Image structure and returns a pointer to the new image.
 %
 %  This method was written and contributed by spd@daphne.cps.unizar.es
 %  (feel free to copy and use it as you want. No warranty).
@@ -1045,7 +1178,6 @@ MagickExport struct dirent *readdir(DIR *entry)
 MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
-#define LoadImageText  "  Loading image...  "
 #define PICTHeaderSize    512
 
   CodecType
@@ -1068,6 +1200,9 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
   long
     y;
 
+  MagickBooleanType
+    status;
+
   PicHandle
     picture_handle;
 
@@ -1099,22 +1234,21 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
     Open image file.
   */
   image=AllocateImage(image_info);
-  status=OpenBlob(image_info,image,ReadBinaryType,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,"Unable to open file",image);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
+  if (status == MagickFalse)
+    return(NULL);
   picture_handle=(PicHandle)
     NewHandle(Max(GetBlobSize(image)-PICTHeaderSize,PICTHeaderSize));
   if (picture_handle == nil)
-    ThrowReaderException(ResourceLimitError,"Unable to allocate memory",
-      image);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   HLock((Handle) picture_handle);
-  (void) ReadBlob(image,PICTHeaderSize,*(char **) picture_handle);
-  status=ReadBlob(image,GetBlobSize(image)-PICTHeaderSize,*(char **) picture_handle);
-  if (status == False)
+  (void) ReadBlob(image,PICTHeaderSize,*(unsigned char **) picture_handle);
+  status=ReadBlob(image,GetBlobSize(image)-PICTHeaderSize,*(unsigned char **)
+    picture_handle);
+  if (status == MagickFalse)
     {
       DisposeHandle((Handle) picture_handle);
-      ThrowReaderException(CorruptImageError,"Unable to read image data",
-        image);
+      ThrowReaderException(CorruptImageError,"UnableToReadImageData");
     }
   GetGWorld(&port,&device);
   theErr=NewGWorld(&graphic_world,0,&(**picture_handle).picFrame,nil,nil,
@@ -1122,8 +1256,7 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
   if ((theErr != noErr) && (graphic_world == nil))
     {
       DisposeHandle((Handle) picture_handle);
-      ThrowReaderException(ResourceLimitError,"Unable to allocate memory",
-        image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   HUnlock((Handle) picture_handle);
   SetGWorld(graphic_world,nil);
@@ -1132,10 +1265,15 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
     {
       DisposeGWorld(graphic_world);
       DisposeHandle((Handle) picture_handle);
-      ThrowReaderException(CorruptImageError,"Unable to read image data",
-        image);
+      ThrowReaderException(CorruptImageError,"UnableToReadImageData");
     }
+#if defined(DISABLE_QUICKTIME)
+  codec='unkn';
+  colormap_id=(-1);
+  depth=picture_info.depth;
+#else
   BottleneckTest(picture_handle,&codec,&depth,&colormap_id);
+#endif
   switch (codec)
   {
     case 'rpza':
@@ -1144,7 +1282,7 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
     case 'raw ':
     case 'smc ':
     {
-      if (depth > 32)
+      if (depth > 200)
         {
           depth-=32;
           picture_info.theColorTable=GetCTable(colormap_id);
@@ -1167,7 +1305,7 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
   image->rows=picture_info.sourceRect.bottom-picture_info.sourceRect.top;
   if ((depth <= 8) && ((*(picture_info.theColorTable))->ctSize != 0))
     {
-      unsigned int
+      unsigned long
         number_colors;
 
       /*
@@ -1180,8 +1318,7 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
             DisposeHandle((Handle) picture_info.theColorTable);
           DisposeGWorld(graphic_world);
           DisposeHandle((Handle) picture_handle);
-          ThrowReaderException(ResourceLimitError,"Unable to allocate memory",
-            image);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
       for (x=0; x < image->colors; x++)
       {
@@ -1217,17 +1354,23 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
     for (x=0; x < image->columns; x++)
     {
       GetCPixel(x,y,&Pixel);
-      q->red=Upscale(Pixel.red & 0xff);
-      q->green=Upscale(Pixel.green & 0xff);
-      q->blue=Upscale(Pixel.blue & 0xff);
+      q->red=ScaleCharToQuantum(Pixel.red & 0xff);
+      q->green=ScaleCharToQuantum(Pixel.green & 0xff);
+      q->blue=ScaleCharToQuantum(Pixel.blue & 0xff);
       if (image->storage_class == PseudoClass)
         indexes[x]=Color2Index(&Pixel);
       q++;
     }
-    if (!SyncImagePixels(image))
+    if (SyncImagePixels(image) == MagickFalse)
       break;
-    if (QuantumTick(y,image->rows))
-      MagickMonitor(LoadImageText,y,image->rows);
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(LoadImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
   }
   UnlockPixels(GetGWorldPixMap(graphic_world));
   SetGWorld(port,device);
@@ -1249,7 +1392,7 @@ MagickExport Image *ReadPICTImage(const ImageInfo *image_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method SearchForFile searches for a file.
+%  SearchForFile() searches for a file.
 %
 %
 */
@@ -1326,6 +1469,7 @@ static Boolean SearchForFile(OSType creator_type,OSType file_type,FSSpec *file,
   return(error == eofErr);
 }
 
+#if !defined(_POSIX_VERSION)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -1337,8 +1481,8 @@ static Boolean SearchForFile(OSType creator_type,OSType file_type,FSSpec *file,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method seekdir sets the position of the next readdir() operation
-%  on the directory stream.
+%  seekdir() sets the position of the next readdir() operation on the directory
+%  stream.
 %
 %  The format of the seekdir method is:
 %
@@ -1359,6 +1503,7 @@ MagickExport void seekdir(DIR *entry,long position)
   assert(entry != (DIR *) NULL);
   entry->d_index=position;
 }
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1371,7 +1516,7 @@ MagickExport void seekdir(DIR *entry,long position)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Method SetApplicationType sets the file type of an image.
+%  SetApplicationType() sets the file type of an image.
 %
 %  The format of the SetApplicationType method is:
 %
@@ -1399,19 +1544,21 @@ MagickExport void SetApplicationType(const char *filename,const char *magick,
 
   Str255
     name;
-   
+
   assert(filename != (char *) NULL);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),filename);
   assert(magick != (const char *) NULL);
   filetype='    ';
-  (void) strncpy((char *) &filetype,magick,Min(strlen(magick),4));
+  (void) CopyMagickString((char *) &filetype,magick,Min(strlen(magick),4));
   if (LocaleCompare(magick,"JPG") == 0)
     (void) strcpy((char *) &filetype,"JPEG");
-  (void) strncpy((char *) name,filename,254);
-  c2pstr((char *) name);
+  c2pstrcpy(name, filename);
   FSMakeFSSpec(0,0,name,&file_specification);
   FSpCreate(&file_specification,application,filetype,smSystemScript);
 }
 
+#if !defined(_POSIX_VERSION)
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -1441,3 +1588,5 @@ MagickExport long telldir(DIR *entry)
   return(entry->d_index);
 }
 #endif
+
+#endif
